'use strict';
angular.module('bamboo.common').service('BlogService', function(ApiService, CommonService, $state, $filter) {
    // service to create one blog
    this.createBlog = function(info, callback) {
      info.action = 'create';
      ApiService.post('/blogs', info).then(function(result) {
        if (result.data.success) {
          CommonService.showNoBlockSuccess('Create Blog Success!');
          if (!callback) {
            $state.reload();
          } else {
            callback(result.data.data);
          }
        } else {
          var _err = result.data.error || '' + result.data.info || '';
          CommonService.showError('Create Blog failed: ' + _err);
        }
      })
    }
    // service to update one blog
    this.updateBlog = function(info, callback) {
      info.action = 'update';

      ApiService.post('/blogs', info).then(function(result) {
        if (result.data.success) {
          CommonService.showNoBlockSuccess($filter('trans')( {eng: 'Update Blog Success!', chn: '博客更新成功！'} ));
          if (!callback) {
            $state.reload();
          } else {
            callback(result.data.data);
          }
        } else {
          var _err = result.data.error || '' + result.data.info || '';
          CommonService.showError('Update Blog failed: ' + _err);
        }
      })
    }
    // service to return blog tabs/category
    this.getBlogTabs = function() {
      return ApiService.blogcategory
    }
    // service to get full infomation of one blog by id(eg, include the comments)
    this.getFullBlog = function(blogid, callback) {
      if (!callback) {
        console.log('Try to get one blog info without callback');
        return;
      }
      ApiService.get('/fullblogs/' + blogid).then(function(result) {
        console.log(result);
        if (result.data.success) {
          callback(result.data.data);
        } else {
          var _err = result.data.error || '' + result.data.info || '';
          CommonService.showError('Get full blog info failed: ' + _err);
        }
      })
    }
    // service to get teacher's blog by userid and support remote page
    this.getTeacherOnePageBlog = function(userid, page, limits, tab, callback) {
      if (!callback) {
        console.log('Try to get teacher blog without callback');
      }
      var info = {
        action: 'getuserblogs',
        page: page, // page number to be loaded, starts from 0
        limit: limits, // items # returns within one page 
        id: userid, // teacher's userid
        tab: tab // blog's category, null for all category 
      };
      ApiService.post('/blogs', info).then(function(result) {
        if (result.data.success) {
          callback(result.data);
        } else {
          var _err = result.data.error || '' + result.data.info || '';
          CommonService.showError('Get posts info failed: ' + _err);
        }
      })
    }
    // service to delete one reply under blog by reply id
    this.delBlogReply = function(replyid) {
      CommonService.confirm({
        message: $filter('trans')({eng:'Are you sure want to delete this reply?',chn:'确定要删除该回复吗？'})
      }).then(function() {
        var params = {
          reply_id: replyid,
        };
        ApiService.post('/blogreply/' + replyid + '/delete', params).then(function(result) {
          if (result.data.success) {
            $state.reload();
          } else {
            var _err = result.data.error || '' + result.data.info || '';
            CommonService.showError('Delete blog reply failed: ' + _err);
          }
        })
      })
    }
    // service to add one reply under one blog
    this.addBlogReply = function(params, callback) {
      if (!params.hasOwnProperty('r_content') || !params.hasOwnProperty('topic_id')) {
        console.log('incompleted params');
        return
      } else {
        ApiService.put('/blogreplies', params).then(function(result) {
          if (result.data.success) {
            if (callback) {
              callback(result.data.data);
            } else {
              $state.reload();
            }
          } else {
            var _err = result.data.error || '' + result.data.info || '';
            CommonService.showError($filter('trans')({
              eng: 'Add blog reply failed:',
              chn: '无法添加回复:'
            }) + _err);
          }
        })
      }
    }
    // service to delete one  blog by id
    this.delBlogById = function(blogid, callback) {
      CommonService.confirm({
        message: $filter('trans')({eng:'Are you sure want to delete this reply?',chn:'确定要删除该回复吗？'})
      }).then(function() {
        var params = {
          id: blogid,
          action: 'delete'
        };
        ApiService.post('/blogs', params).then(function(result) {
          if (result.data.success) {
            if (callback) {
              callback(result.data.data);
            } else {
              $state.reload();
            }
          } else {
            var _err = result.data.error || '' + result.data.info || '';
            CommonService.showError('Delete blog failed: ' + _err);
          }
        })
      })
    }

    this.BlogApi = function(info, callback) {
      function handleData(data) {
        if (!callback) {
          CommonService.showNoBlockSuccess($filter('trans')({ eng : 'Update Successful!', chn : '更新成功'}));
          $state.reload();
        } else {
          callback(data);
        }
      }
      ApiService.post('/blogs', info).then(function(result) {
        if (result.data.success) {
          handleData(result.data);
        } else {
          CommonService.showError($filter('trans')({ eng : 'Update Fail!', chn : '无法更新！'}) + result.data.error);
          console.log(result);
        }
      });
    };
    this.FeedbackApi = function(info, callback) {
      function handleData(data) {
        if (!callback) {
          CommonService.showNoBlockSuccess($filter('trans')({ eng : 'Update Successful!', chn : '更新成功'}));
          $state.reload();
        } else {
          callback(data.data);
        }
      }
      ApiService.post('/feedbacks', info).then(function(result) {
        if (result.data.success) {
          handleData(result.data);
        } else {
          CommonService.showError($filter('trans')({ eng : 'Update Fail!', chn : '无法更新！'}) + result.data.error);
          console.log(result);
        }
      });
    };
})