'use strict';
angular.module('bamboo.common')
  .filter('notEmpty', function() {
    var bar;
    return function(obj) {

      // null and undefined are "empty"
      if (obj == null) return false;

      // Assume if it has a length property with a non-zero value
      // that that property is correct.
      if (obj.length > 0) return true;
      if (obj.length === 0) return false;

      // Otherwise, does it have any properties of its own?
      // Note that this doesn't handle
      // toString and valueOf enumeration bugs in IE < 9
      for (bar in obj) {
        if (obj.hasOwnProperty(bar)) {
          return true;
        }
      }
      return false;
    };
  }).filter('isEmpty', function() {
    var bar;
    return function(obj) {

      // null and undefined are "empty"
      if (obj == null) return true;

      // Assume if it has a length property with a non-zero value
      // that that property is correct.
      if (obj.length > 0) return false;
      if (obj.length === 0) return true;

      // Otherwise, does it have any properties of its own?
      // Note that this doesn't handle
      // toString and valueOf enumeration bugs in IE < 9
      for (bar in obj) {
        if (obj.hasOwnProperty(bar)) {
          return false;
        }
      }
      return true;
    };
  }).filter('htmlToPlaintext', function() {
    return function(text) {
      return text ? String(text).replace(/<[^>]+>/gm, '') : '';
    };
  }).filter('chatDateFilter', function($filter) {
    return function(text) {
      console.log(text);
      var today = new Date();
      var yesterday = new Date();
      var todayStr = today.toDateString();
      yesterday.setDate(yesterday.getDate() - 1);
      var yesterdayStr = yesterday.toDateString();
      var textDateStr = new Date(text).toDateString();
      //yesterday  -- Yesterday
      //other day -- 8/5/2017 10:50 PM
      //console.log($filter('date')(text, "yyyy-MM-dd"));
      if (textDateStr === todayStr) {
        return $filter('date')(text, "h:mm a");
      } else if (textDateStr === yesterdayStr) {
        return 'Yesterday ' + $filter('date')(text, "h:mm a");
      } else {
        return $filter('date')(text, "MM/dd/yyyy h:mm a");
      }
    };
  }).filter('chatDateFilter2', function($filter) {
    return function(text) {
  //    console.log(text);
      var today = new Date();
      var yesterday = new Date();
      var todayStr = today.toDateString();
      yesterday.setDate(yesterday.getDate() - 1);
      var yesterdayStr = yesterday.toDateString();
      var textDateStr = new Date(text).toDateString();
      //yesterday  -- Yesterday
      //other day -- 8/5/2017 10:50 PM
      //console.log($filter('date')(text, "yyyy-MM-dd"));
      if (textDateStr === todayStr) {
        return $filter('date')(text, "h:mm a");
      } else if (textDateStr === yesterdayStr) {
        return 'Yesterday ';
      } else {
        return $filter('date')(text, "MM/dd/yyyy");
      }
    };
  }).filter('userAvatarImage', function(CommonService) {
    return function(input) {
        return CommonService.getAvatarSrc(input);
    }
  }).filter('secondsToHms', function(CommonService) {
    return function(input) {
        return CommonService.secondsToHms(input);
    }
  });

angular.module('ionToggleText', ['ionic'])
  .directive('ionToggleText', function() {

    var $ = angular.element;

    return {
      restrict: 'A',
      link: function($scope, $element, $attrs) {

        // Try to figure out what text values we're going to use

        var textOn = $attrs.ngTrueValue || 'on',
          textOff = $attrs.ngFalseValue || 'off';

        if ($attrs.ionToggleText) {
          var x = $attrs.ionToggleText.split(';');

          if (x.length === 2) {
            textOn = x[0] || textOn;
            textOff = x[1] || textOff;
          }
        }

        // Create the text elements

        var $handleTrue = $('<div class="handle-text handle-text-true">' + textOn + '</div>'),
          $handleFalse = $('<div class="handle-text handle-text-false">' + textOff + '</div>');

        var label = $element.find('label');

        if (label.length) {
          label.addClass('toggle-text');

          // Locate both the track and handle elements

          var $divs = label.find('div'),
            $track, $handle;

          angular.forEach($divs, function(div) {
            var $div = $(div);

            if ($div.hasClass('handle')) {
              $handle = $div;
            } else if ($div.hasClass('track')) {
              $track = $div;
            }
          });

          if ($handle && $track) {

            // Append the text elements

            $handle.append($handleTrue);
            $handle.append($handleFalse);

            // Grab the width of the elements

            var wTrue = $handleTrue[0].offsetWidth,
              wFalse = $handleFalse[0].offsetWidth;

            // Adjust the offset of the left element

            $handleTrue.css('left', '-' + (wTrue + 10) + 'px');

            // Ensure that the track element fits the largest text

            var wTrack = Math.max(wTrue, wFalse);
            $track.css('width', (wTrack + 60) + 'px');
          }
        }
      }
    };
  });
