'use strict';

angular.module('bamboo.common').service('CourseService', function($state, $stateParams, $rootScope, ApiService, CommonService, $filter){
  /**
   * @ngdoc method
   * @name $course.service#getCoursesWithSorting
   * @kind function
   *
   * @description Get courses based on the provided page, sorting method and page size.
   *
   * @param {int} the page number for backend to retrieve
   * @param {string} The sorting method. it can be the following values:
   *	visitercounter: sort based on the visiter counts
   *	position: sorting based on recommended by admin
   *	score: sorting based on the score feedback by user
   *	update: sorgint based on the updated date
   *	created: sorting based on the created date
   *	recommend: sorting based on the recommendations by user
   * @param {object} how many items to return for this page
   *
   * @return {object} the list course objects. if failed to retrieve, it will return null
   */

  // sercive to get courses info by ids
  this.getCoursesInfoByIds = function(cidsArray, callback) {
    if (!callback) {
      console.log('Try to get courses info without callback!');
      return;
    }
    var info = {
      ids: cidsArray,
      action: 'getCourseByIds'
    };
    ApiService.post('/lcourses', info).then(function(result) {
      if (result.data.success) {
        callback(result.data.data);
      } else {
        var _err = result.data.error || '' + result.data.info || '';
        CommonService.showError('Get courses info failed: ' + _err);
      }
    });
  };

    // service to get courseware info by ids
  this.getCoursewareInfoByIds = function(ids, callback) {
    if (!callback) {
      console.log('Try to get courseware info without callback!');
    }
    var info = {
      ids: ids,
      action: 'getcoursesbyids'
    };
    ApiService.post('/mvsubjects', info).then(function(result) {
      if (result.data.success) {
        callback(result.data.data);
      } else {
        var _err = result.data.error || '' + result.data.info || '';
        CommonService.showError($filter('trans')({
            eng: 'Get courseware info failed: ',
            chn: '无法获取课程章节信息： '
          }) + _err);
      }
    })
  };

  this.post = function(url, info,callback) {
    ApiService.post(url, info).then(function(result) {
      if (result.data.success) {
        if(callback){
          callback(result.data.data);
        }else{
          CommonService.showInfoWithTitle('Status','Update Successful!');
          $state.reload();
        }
      } else {
        CommonService.showError('Update Fail!');
        console.log(result.data);
      }
    });
  };

});
