'use strict'
angular.module('bamboo.common').service('ExamService', function(ApiService, CommonService) {
  // service to get full info about one paper by id
  this.getFullPaperById = function(paperid, callback) {
    //log warning if no callback
    if (!callback) {
      console.log('Try to get full paper(id:' + paperid + ') without callback');
    }
    var info = {
      id: paperid,
      action: 'getfullpaper'
    };
    ApiService.post('/exam', info).then(function(result) {
      if (result.data.success) {
        callback(result.data.data);
      } else {
        var error = result.data.error || '' + result.data.info || '';
        //CommonService.showError("Error : " + error);
        CommonService.showError("Error : " + error)
      }
    })
  }

  function patchQResult(q) {
    if (q.type == 'mquestion') return;
    if (!q.result && !q.iscore && !q.resultFlag && !q.files) {
      q.NOT_ATTEMPTED = true;
    } else if (!q.resultFlag && q.type != 'short' && q.result) {
      q.WRONG = true;
    }
  }

  function calBankResult(q) {
    var answerd = false;
    var correct = false;
    var needReview = false;
    var userScore = 0;
    var userAnswer = [];
    switch (q.type) {
      case "single":
        angular.forEach(q._options, function(opt, index) {
          if (index == q.select && opt.correct) {
            correct = true;
            userScore = q.score;
            userAnswer.push(opt.text);
          }
        })
        if (q.select || q.select == 0) {
          answerd = true;
        }
        break;
      case 'short':
        if (q.selection) {
          userAnswer.push(q.selection);
          needReview = true;
          answerd = true;
        }
        break;
      case "multiple":
     // case "multiselect":
        correct = true;
        var subcorrect = 0;
        for (var j = 0; j < q._options.length; j += 1) {
          var opt = q._options[j];
          //  console.log(opt);
          if (opt.selection) {
            answerd++;
            userAnswer.push(opt.text);
            answerd = true;
          }
          if ((opt.selection && opt.correct) || (!opt.selection && !opt.correct)) {
            subcorrect++;
          } else {
            correct = false;
            console.log('wrong selection');
          }
        }
        if (!answerd) {
          correct = false;
        }
        // if (q.individual) {
        //   userScore = (subcorrect * q.score) / (q._options.length || 1)
        if (correct) {
          userScore = q.score;
        }
        break;
      case "multiselect":
        correct = true;
        var subcorrect = 0;
        var subwrong=0;

        for (var j = 0; j < q._options.length; j += 1) {
          var opt = q._options[j];
            console.log(opt);
          if (opt.selection) {
            answerd++;
            userAnswer.push(opt.text);
            answerd = true;
          }
          if (opt.selection && opt.correct){
            subcorrect++;
          } else if(opt.selection && !opt.correct) {
            subwrong++;
            correct = false;
          } else if(!opt.selection && opt.correct) {
           // subwrong++;
            correct = false;
            console.log('ignore selection');
          }
        }
        if (!answerd) {
          correct = false;
        }
        var _qscore=0;
        if(correct){
          _qscore=q.score;
        }else{
           _qscore=subcorrect-subwrong*0.5;
        }
        if(_qscore<0){
          _qscore=0;
        }
        if(_qscore>q.score){
         _qscore=q.score;
        }
        console.log(correct,subcorrect,subwrong,_qscore)
        userScore=_qscore;
        break;
      case "tof":
        if (!q.selection && q.selection != false) {
          // q.judgement = 'Not Attempted';
        } else {
          answerd = true;
          userAnswer.push(q.selection);
          if (q.selection == true && q._options[0].correct) {
            userScore = q.score;
            correct = true;
          } else if (q.selection == false && !q._options[0].correct) {
            correct = true;
            userScore = q.score;
          }
        }
        break;
      case "typing":
        if (q.correctRate >= 0.6) {
          correct = true;
        }
        userScore = q.score * q.correctRate;
        if (q.result) {
          userAnswer.push(q.result);
        }
        break;
      case "mfill":
        correct = true;
        var subcorrect = 0;
        angular.forEach(q._options, function(opt, idx) {
          if (opt.selection) {
            answerd = true;
            var _result = false;
            for (var n = 0; n < opt.texts.length; n++) {
              if (opt.selection.split(' ').join('') == opt.texts[n].text.split(' ').join('')) {
                _result = true;;
                opt.correct = true;
                break;
              }
            }
            if (_result) {
              subcorrect++;
            } else {
              correct = false;
              console.log(opt);
            }
            userAnswer.push(opt.selection);
          } else {
            correct = false;
          }
        });
        if (correct) {
          userScore = q.score;
        } else if (q.individual) {
          var _qlen = q._options.length || 1;
          userScore = (q.score * subcorrect) / _qlen;
        }
        break;
    }
    /*     var answerd = false;
     var correct = false;
     var needReview=false;
     var userScore=0;
     var userAnswer=[];*/
    var result = {
      answerd: answerd,
      correct: correct,
      needReview: needReview,
      userScore: userScore,
      userAnswer: userAnswer,
    }
    return result;
  }
  this.calQuestionResult=calBankResult;

  var newseq = [];
  var _singleQ = [];
  var _multipleQ = [];
  var _multipleSQ = [];
  var _tofQ = [];
  var _fillQ = [];
  var _mfillQ = [];
  var _shortQ = [];
  var _mquestionQ = [];
  var _multiselectQ = [];


  this.prepareQuestions = function(data, sortFlag) {
    var _qsts = {};
    console.log(data);
    var exam = angular.copy(data.paper);
    newseq = [];
    _singleQ = [];
    _multipleQ = [];
    _multipleSQ = [];
    _tofQ = [];
    _fillQ = [];
    _mfillQ = [];
    _shortQ = [];
    _mquestionQ = [];
    angular.forEach(data.qstList, function(fullquestion, idx) {
      var id = fullquestion._id;
      _qsts[id] = fullquestion;
      if (fullquestion.audio) {
        fullquestion.audiourl = $sce.trustAsResourceUrl(self.audioPath + encodeURI(fullquestion.audio));
      }
      if (fullquestion.video) {
        fullquestion.videourl = $sce.trustAsResourceUrl(self.videoPath + encodeURI(fullquestion.video));
      }
      if (fullquestion.tip && !fullquestion.tips) {
        fullquestion.tips = [];
        fullquestion.tips.push({
          text: fullquestion.tip
        });
      }
      //fullquestion.options = [];
      fullquestion = convertBankQ(fullquestion);
      if (fullquestion.type == "single" || fullquestion.type == "multiple"|| fullquestion.type == "multiselect") {
        fullquestion._options.sort(function(a, b) {
          return Math.random() > .5 ? -1 : 1;
        });
        fullquestion._options.sort(function(a, b) {
          return Math.random() > .5 ? -1 : 1;
        });
      }
    });
    var scorsInfo = {};
    angular.forEach(exam.questions, function(val, index) {
      if (val.id) {
        scorsInfo[val.id] = val;
      }
    })
    exam.totalScore = 0;
    var idx = 0;
    angular.forEach(exam.questions, function(question, index) {
      if (question.id && _qsts[question.id]) {
        var fullquestion = _qsts[question.id];
        if (fullquestion.type == "info") {
          addQinQue(fullquestion, sortFlag);
        } else if (!fullquestion.root && question.type != 'mquestion' && question.type != 'info') {
          fullquestion.score = parseInt(question.score || 0);
          exam.totalScore += fullquestion.score || 0;
          idx++;
          //fullquestion.newIdx = fullquestion.q_index = idx;
          console.log(idx);
          addQinQue(fullquestion, sortFlag);
          //_questions.push(fullquestion);
        } else if (question.type == 'mquestion') {
          if (fullquestion.questions && fullquestion.questions.length) {
            angular.forEach(fullquestion.questions, function(id) {
              if (_qsts[id]) {
                var subq = _qsts[id];
                if (!scorsInfo[id]) {
                  scorsInfo[id] = {};
                }
                subq.score = parseInt(scorsInfo[id].score || 0);
                subq.rootContent = fullquestion.content;
                if (fullquestion.audiourl) {
                  subq.rootAudioURL = fullquestion.audiourl;
                }
                if (fullquestion.videourl) {
                  subq.rootVideoURL = fullquestion.videourl;
                }
                exam.totalScore += subq.score;
                idx++;
                //subq.newIdx = subq.q_index = idx;
                // _questions.push(subq);
                addMQinQue(subq, sortFlag);
              }
            })
          }
        }
      }
    });
    getFullQue(sortFlag);
    delete exam['questions'];
    exam.questions = newseq;
    sortQList(exam.questions, _qsts);
    console.log(newseq);
    return exam;
  }

  function sortQList(list, qMap) {
    var qidx = 0;
    angular.forEach(list, function(q, idx) {
      var fullquestion = qMap[q._id];
      //  console.log(fullquestion);
      //not mq,not info, not subq
      if (!fullquestion.root && q.type != 'mquestion' && q.type != 'info') {
        fullquestion.newIdx = fullquestion.q_index = (++qidx);
      }
      //subq
      else if (fullquestion.root) {
        // console.log(fullquestion);
        fullquestion.newIdx = fullquestion.q_index = (++qidx);
      }
      //info
      else if (q.type == 'info') {}
    });
    console.log(list);
  }

  function getFullQue(sortFlag) {
    if (sortFlag) {
      newseq = newseq.concat(_tofQ);
      newseq = newseq.concat(_singleQ);
      newseq = newseq.concat(_mfillQ);
      newseq = newseq.concat(_multipleQ);
      newseq = newseq.concat(_multipleSQ);
      newseq = newseq.concat(_multiselectQ);
      newseq = newseq.concat(_shortQ);
      newseq = newseq.concat(_mquestionQ);
      return newseq;
    } else {
      return newseq;
    }
  }
  function convertBankQ(q) {
    if (q.ver && q.ver == 1) {
      if (q.htmlFlag && q.type == 'mfill') {
        handleEquation(q);
      }
      return q;
    }
    //  console.log(q);
    q.ver = 1;
    q._options = [];
    switch (q.type) {
      case 'single':
      case 'multiple':
      case 'multiselect':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            text: text,
            correct: true,
          }
          q._options.push(info);
        })
        angular.forEach(q.references, function(text, index) {
          var info = {
            text: text,
          }
          q._options.push(info);
        })
        break;
      case 'fill':
      case 'mfill':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            texts: [{
              text: text
            }],
          }
          q._options.push(info);
        })
        q.type = 'mfill'; // force to change fill to mfill
        if (q.htmlFlag) {
          handleEquation(q);
        }
        break;
      case 'tof':
        if (q.answer) {
          var _t = false;
          if (q.answer[0] && (q.answer[0] == true || q.answer[0] == 'true')) {
            _t = true;
          }
          var info = {
            correct: _t,
          }
          q._options.push(info);
        }
        break;
    }
    delete q.answer;
    delete q.references;
    delete q.options;
    return q;
  }
  function addQinQue(fullquestion, sortFlag) {
    //enable dragFlag for mDD and sDD question
    if (fullquestion.type === 'mDD' || fullquestion.type === 'sDD') {
      fullquestion.dragFlag = true;
    } else {
      fullquestion.dragFlag = false;
    }
    if (sortFlag) {
      switch (fullquestion.type) {
        case "tof":
          _tofQ.push(fullquestion);
          break;
        case "sDD":
        case "single":
          _singleQ.push(fullquestion);
          break;
        case "mDD":
        case "fill":
        case "mfill":
          _mfillQ.push(fullquestion);
          break;
        case "multiple":
          _multipleQ.push(fullquestion);
          break;
        case "multiselect":
          _multipleSQ.push(fullquestion);
          break;
        case "short":
          _shortQ.push(fullquestion);
          break;
        default:
          console.log("---- !!! Unknow type: " + fullquestion.type);
          break;
      }
    } else {
      newseq.push(fullquestion);
    }
  }
  this.calResult = function(exam) {
    console.log(" ---calResult");
    var _result = {
      questions: {},
      counter: 0,
      correct: 0,
      wrong: 0,
      miss: 0,
      score: 0,
    };
    var _knowledgeresult = {};
    var _levelResult = {};
    var _typeResult = {};
    var correctQs = [];
    var wrongQs = [];
    var counter = 0;
    var errorqsts = [];
    var needreviewflag = false;
    //  console.log(exam.questions);
    for (var idx = 0; idx < exam.questions.length; idx++) {
      var q = exam.questions[idx];
     // q.index=idx;
      if (q.type == 'info') {
        continue;
      }
      var ok_state = false;
      counter++;
      var q_result = calBankResult(q);
      console.log(q_result);
      /*var result = {
        answerd: answerd,
        correct: correct,
        needReview: needReview,
        userScore: userScore,
        userAnswer: userAnswer,
      }*/
      if (q_result.correct) {
        _result.correct++;
        correctQs.push(q._id);
        // q.correct=true;
      } else if (!q_result.answerd || !q_result.correct) {
        errorqsts.push(q._id);
        if (!q_result.answerd) {
          _result.miss++;
        } else {
          _result.wrong++;
        }
        wrongQs.push(q._id);
      }
      _result.score += q_result.userScore || 0;
      console.log(_result.score,q_result.userScore,idx);
      q.resultFlag = q_result.correct;
      q.answerd = q_result.answerd;
      // console.log(q.resultFlag);
      var knowledge = q.knowledge || 'other';
      /* if (knowledge && !_knowledgeresult[knowledge]) {
         _knowledgeresult[knowledge] = {
           correct: 0,
           wrong: 0,
         }
       }*/
      if (knowledge) {
        if (!_knowledgeresult[knowledge]) {
          _knowledgeresult[knowledge] = {
            correct: 0,
            wrong: 0,
            corCount: 0,
            items: 1,
          }
        } else {
          _knowledgeresult[knowledge].items++;
        }
      }
      var level = q.level || 0;
      if (!_levelResult[level]) {
        _levelResult[level] = {
          correct: 0,
          wrong: 0,
        }
      }
      if (!_knowledgeresult[knowledge][level]) {
        _knowledgeresult[knowledge][level] = {
          correct: 0,
          wrong: 0,
        }
      }
      var type = q.type || 'other';
      if (!_typeResult[type]) {
        _typeResult[type] = {
          correct: 0,
          wrong: 0,
        }
      }
      if (q_result.correct) {
        if (knowledge) {
          _knowledgeresult[knowledge].correct += q.score;
          _knowledgeresult[knowledge].corCount++;
          _knowledgeresult[knowledge][level].correct += q.score;
        }
        _levelResult[level].correct += q.score;
        _typeResult[type].correct += q.score;
      } else {
        if (knowledge) {
          _knowledgeresult[knowledge].wrong += q.score;
          _knowledgeresult[knowledge][level].wrong += q.score;
        }
        _levelResult[level].wrong += q.score;
        _typeResult[type].wrong += q.score;
      }
      var info = {
        flag: q_result.correct,
        score: q_result.userScore || 0,
        idx:idx,
        needreview: q_result.needReview,
        userAnswer: q_result.userAnswer,
        answerd: q_result.answerd,
      };
      if (info.needreview) {
        info.maxScore = q.score;
      }
      if (!info.flag && q_result.userScore) {
        info.iscore = q_result.userScore;
      }
      _result.questions[q._id] = info;
      if (q.type == 'short') {
        // console.log(_result.questions[q._id]);
        if (q.files && q.files.length > 0) {
          if (!_result.questions[q._id]) {
            _result.questions[q._id] = {};
          }
          _result.questions[q._id].attachments = q.files;
        }
        //  console.log(q);
        // console.log(_result.questions[q._id]);
      }
      if (q_result.needReview) {
        _result.needreview = true;
      }
      patchQResult(q);
    };
    console.log(needreviewflag);
    exam.get_score = _result.score;
    console.log(_result);
    _result.correctQs = correctQs;
    _result.wrongQs = wrongQs;
    _result.knowledgeresult = _knowledgeresult;
    _result.levelresult = _levelResult;
    _result.typeresult = _typeResult;
    _result.totalscores = exam.totalScore;
    return _result;
  };
  this.convertQ = function(q) {
    if (q.content) {
      q.content = q.content.replace('src="//', 'src="http://');
      q.content = q.content.replace('src="//', 'src="http://');
      angular.forEach(q._options, function(opt) {
        if (opt.text) {
          opt.text = opt.text.replace('src="//', 'src="http://');
          opt.text = opt.text.replace('src="//', 'src="http://');
        }
      })
    }
    if (q.ver && q.ver == 1) {
      return q;
    }
    console.log(q);
    q.ver = 1;
    q._options = [];
    switch (q.type) {
      case 'single':
        if (q.options && q.options.length) {
          angular.forEach(q.options, function(opt, index) {
            var info = opt;
            if (opt.text == q.ok_answer) {
              info.correct = true;
            }
            q._options.push(info);
          })
        } else {
          angular.forEach(q.answer, function(text, index) {
            var info = {
              text: text,
              correct: true,
            }
            q._options.push(info);
          })
          angular.forEach(q.references, function(text, index) {
            var info = {
              text: text,
            }
            q._options.push(info);
          })
        }
        delete q.ok_answer;
        break;
      case 'multiple':
      case 'multiselect':
        if (q.options && q.options.length) {
          angular.forEach(q.options, function(opt, index) {
            q._options.push(opt);
          })
        } else {
          angular.forEach(q.answer, function(text, index) {
            var info = {
              text: text,
              correct: true,
            }
            q._options.push(info);
          })
          angular.forEach(q.references, function(text, index) {
            var info = {
              text: text,
            }
            q._options.push(info);
          })
        }
        // console.log(q._options);
        break;
      case 'fill':
      case 'mfill':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            texts: [{
              text: text
            }],
          }
          q._options.push(info);
        })
        angular.forEach(q.options, function(val, index) {
          var _text = {
            text: val.text,
          }
          var info = {
            texts: [_text],
          }
          q._options.push(info);
        })
        delete q.options;
        // console.log(q._options);
        break;
      case 'tof':
        if (q.tof_answer) {
          var info = {
            correct: q.tof_answer,
          }
          q._options.push(info);
          delete q.tof_answer;
        } else if (q.answer) {
          var _t = false;
          if (q.answer[0] && (q.answer[0] == true || q.answer[0] == 'true')) {
            _t = true;
          }
          var info = {
            correct: _t,
          }
          q._options.push(info);
        }
        // console.log(q);
        break;
    }
    delete q.answer;
    delete q.references;
    delete q.options;
    return q;
  }
  this.convertBankQ = function(q) {
    console.log(ApiService.subdomain);
    if (q.content&&(q.content.indexOf('src="//')>-1)) {
      q.content = q.content.replace('src="//', 'src="http://');
      q.content = q.content.replace('src="//', 'src="http://');
    }
    angular.forEach(q._options, function(opt) {
      if (opt.text&&(opt.text.indexOf('src="//')>-1)) {
        opt.text = opt.text.replace('src="//', 'src="http://');
        opt.text = opt.text.replace('src="//', 'src="http://');
      }
    })
    if (q.ver && q.ver == 1) {
      return q;
    }
    //  console.log(q);
    q.ver = 1;
    q._options = [];
    switch (q.type) {
      case 'single':
      case 'multiple':
      case 'multiselect':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            text: text,
            correct: true,
          }
          q._options.push(info);
        })
        angular.forEach(q.references, function(text, index) {
          var info = {
            text: text,
          }
          q._options.push(info);
        })
        break;
      case 'fill':
      case 'mfill':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            texts: [{
              text: text
            }],
          }
          q._options.push(info);
        })
        q.type = 'mfill'; // force to change fill to mfill
        break;
      case 'tof':
        if (q.answer) {
          var _t = false;
          if (q.answer[0] && (q.answer[0] == true || q.answer[0] == 'true')) {
            _t = true;
          }
          var info = {
            correct: _t,
          }
          q._options.push(info);
        }
        break;
    }
    delete q.answer;
    delete q.references;
    delete q.options;
    if (ApiService.subdomain == 'xijing') {
      angular.forEach(q._options, function(opt) {
        if (opt.text) {
          opt.text = opt.text.replace("course.xijing.edu.cn:3500", "media.xijing.edu.cn");
          opt.text = opt.text.replace("course.xijing.edu.cn:3500", "media.xijing.edu.cn");
        }
      })
    }
    return q;
  }
  // service to update paper
  this.updatePaperById = function(params, callback) {
    if (!params.hasOwnProperty('_id')) {
      console.log('try to update papar without id');
      return;
    }
    params.action = 'updatepaper';
    ApiService.post('/exam', params).then(function(result) {
      if (result.data.success) {
        if (callback) {
          callback(result.data.data);
        }
      } else {
        var error = result.data.error || '' + result.data.info || '';
        CommonService.showError("Error : " + error)
      }
    })
  }
  // service to update dpaper
  this.updateDpaperById = function(params, callback) {
    if (!params.hasOwnProperty('id')) {
      console.log('try to update papar without id');
      return;
    }
    params.action = 'updatedpaper';
    ApiService.post('/exam', params).then(function(result) {
      if (result.data.success) {
        if (callback) {
          callback(result.data.data);
        }
      } else {
        var error = result.data.error || '' + result.data.info || '';
        CommonService.showError("Error : " + error);
      }
    })
  }
  this.convertV1Question = function(q) {
    if (q.ver || q.ver >= 1) {
      return q;
      // no need to convert
    }
    console.log("version ");
    q.ver = 1;
    q._options = [];
    console.log(q);
    switch (q.type) {
      case 'single':
        if (q.options && q.options.length) {
          angular.forEach(q.options, function(opt, index) {
            var info = opt;
            if (opt.text == q.ok_answer) {
              info.correct = true;
            }
            q._options.push(info);
          })
        } else {
          angular.forEach(q.answer, function(text, index) {
            var info = {
              text: text,
              correct: true,
            }
            q._options.push(info);
          })
          angular.forEach(q.references, function(text, index) {
            var info = {
              text: text,
            }
            q._options.push(info);
          })
        }
        // delete q.ok_answer;
        break;
      case 'multiple':
      case 'multiselect':
        if (q.options && q.options.length) {
          angular.forEach(q.options, function(opt, index) {
            q._options.push(opt);
          })
        } else {
          angular.forEach(q.answer, function(text, index) {
            var info = {
              text: text,
              correct: true,
            }
            q._options.push(info);
          })
          angular.forEach(q.references, function(text, index) {
            var info = {
              text: text,
            }
            q._options.push(info);
          })
        }
        break;
      case 'fill':
      case 'mfill':
        angular.forEach(q.answer, function(text, index) {
          var info = {
            texts: [{
              text: text
            }],
          }
          q._options.push(info);
        })
        angular.forEach(q.options, function(val, index) {
          var _text = {
            text: val.text,
          }
          var info = {
            texts: [_text],
          }
          q._options.push(info);
        })
        break;
      case 'tof':
        if (q.tof_answer) {
          var info = {
            correct: q.tof_answer,
          }
          q._options.push(info);
          delete q.tof_answer;
        } else if (q.answer) {
          var _t = false;
          if (q.answer[0] && (q.answer[0] == true || q.answer[0] == 'true')) {
            _t = true;
          }
          var info = {
            correct: _t,
          }
          q._options.push(info);
        }
        // console.log(q);
        break;
    }
    // delete q.answer;
    // delete q.references;
    // delete q.options;
    return q;
  }
  this.api = function(info, callback) {
    ApiService.post('/exam', info).then(function(result) {
      if (result.data.success) {
        if (!callback) {
          CommonService.showNoBlockSuccess('Update Successful!');
          $state.reload();
        } else {
          callback(result.data.data);
        }
      } else {
        var error = result.data.error || '' + result.data.info || '';
        CommonService.showError("Error : " + error)
      }
    })
  }
});
