angular.module('bamboo.common').provider('loginService', function() {
  var userToken = sessionStorage.getItem('userToken'),
    errorState = 'app.home',
    logoutState = 'app.home';
  this.$get = function($rootScope, $http, $q, $location, $state, ApiService, $localstorage, CommonService, $filter) {
    /**
     * Low-level, private functions.
     */
    var setToken = function(token) {
      if (!token) {
        sessionStorage.removeItem('userToken');
      } else {
        sessionStorage.setItem('userToken', token);
      }
    };
    var getLoginData = function(oldDoamin) {
      userToken = oldDoamin || userToken;
      var subDomain = ApiService.subdomain;
      wrappedService.loggedDomain = true;
      wrappedService.subDomain = subDomain;
    };
    var managePermissions = function() {
      // Register routing function.
      $rootScope.$on('$stateChangeStart', function(event, to, toParams, from, fromParams) {
        /**
         * $stateChangeStart is a synchronous check to the accessLevels property
         * if it's not set, it will setup a pendingStateChange and will let
         * the grandfather resolve do his job.
         *
         * In short:
         * If accessLevels is still undefined, it let the user change the state.
         * Grandfather.resolve will either let the user in or reject the promise later!
         */
        if (wrappedService.userRole === null) {
          wrappedService.doneLoading = false;
          wrappedService.pendingStateChange = {
            to: to,
            toParams: toParams
          };
          return;
        }
        // if the state has undefined accessLevel, anyone can access it.
        // NOTE: if `wrappedService.userRole === undefined` means the service still doesn't know the user role,
        // we need to rely on grandfather resolve, so we let the stateChange success, for now.
        if (to.accessLevel === undefined || to.accessLevel.bitMask & wrappedService.userRole.bitMask) {
          angular.noop(); // requested state can be transitioned to.
        } else {
          event.preventDefault();
          $rootScope.$emit('$statePermissionError');
          $state.go(errorState, {
            error: 'unauthorized'
          }, {
            location: false,
            inherit: false
          });
        }
      });
      /**
       * Gets triggered when a resolve isn't fulfilled
       * NOTE: when the user doesn't have required permissions for a state, this event
       *       it's not triggered.
       *
       * In order to redirect to the desired state, the $http status code gets parsed.
       * If it's an HTTP code (ex: 403), could be prefixed with a string (ex: resolvename403),
       * to handle same status codes for different resolve(s).
       * This is defined inside $state.redirectMap.
       */
      $rootScope.$on('$stateChangeError', function(event, to, toParams, from, fromParams, error) {
        /**
         * This is a very clever way to implement failure redirection.
         * You can use the value of redirectMap, based on the value of the rejection
         * So you can setup DIFFERENT redirections based on different promise errors.
         */
        var errorObj, redirectObj;
        // in case the promise given to resolve function is an $http request
        // the error is a object containing the error and additional informations
        error = error && (typeof error === 'object') && error.status ? error.status.toString() : error;
        // in case of a random 4xx/5xx status code from server, user gets loggedout
        // otherwise it *might* forever loop (look call diagram)
        if (/^[45]\d{2}$/.test(error)) {
          wrappedService.logoutUser();
        }
        /**
         * Generic redirect handling.
         * If a state transition has been prevented and it's not one of the 2 above errors, means it's a
         * custom error in your application.
         *
         * redirectMap should be defined in the $state(s) that can generate transition errors.
         */
        if (angular.isDefined(to.redirectMap) && angular.isDefined(to.redirectMap[error])) {
          if (typeof to.redirectMap[error] === 'string') {
            return $state.go(to.redirectMap[error], {
              error: error
            }, {
              location: false,
              inherit: false
            });
          } else if (typeof to.redirectMap[error] === 'object') {
            redirectObj = to.redirectMap[error];
            return $state.go(redirectObj.state, {
              error: redirectObj.prefix + error
            }, {
              location: false,
              inherit: false
            });
          }
        }
        return $state.go(errorState, {
          error: error
        }, {
          location: false,
          inherit: false
        });
      });
    };
    /**
     * High level, public methods
     */
    var wrappedService = {
      getMyProfile: function(force) {
        var deferred = $q.defer();
        console.log("--- get my profile ---");
        var self = this;
        if (this.profile&&this.user && !force) {
          console.log("--- 2---");
          deferred.resolve(this.profile);
        } if(!this.user&&!!force){
          deferred.resolve({});
        }else {
          ApiService.get('/profile/').then(function(result) {
            console.log(result);
            if (result.data.success) {
              console.log(result.data);
              self.profile = result.data.data;
              console.log("--- 3---");
              deferred.resolve(self.profile);
            } else {
              console.log("--- 4---");
              deferred.reject(null);
            }
          });
        }
        return deferred.promise;
      },
      getLoginInfo: function(force) {
        var self = this;
        if (this.school) {
          force = force || false;
          if (!force&&this.user) {
            return $q.when(this.user);
          } else {
            return ApiService.post('/loginstatus').then(function(result) {
              //console.log(result);

              if (result.data.success) {
                self.user = result.data.data;
                //  console.log(self.user);
              } else {
                self.user = null;
              }
              return $q.when(self.user);
            });
          }
        } else {
          return ApiService.getSchoolInfo(this.subDomain).then(function(school) {
            self.school = school;
            // self.forumcategoryLists=ApiService.forumcategoryKeys;
            $rootScope.currentSchool = school;
            return ApiService.post('/loginstatus').then(function(result) {
              //console.log(result);

              if (result.data.success) {
                self.user = result.data.data;
                //   console.log(self.user);
              } else {
                self.user = null;
              }
              return $q.when(self.user);
            });
          }, function(err) {
            console.log(" School Service Retry counter : " + $rootScope.apiRetry);
            if (!$rootScope.apiRetry) {
              $rootScope.apiRetry = 1;
              $state.go('app.home');
            } else if ($rootScope.apiRetry < 4) {
              $rootScope.apiRetry++;
              $state.go('app.home');
            } else {
              $state.go('errstate');
            }
          });
        }
      },
      login: function(loginInfo) {
        var self = this;
        var deferred = $q.defer();
        loginInfo.mobile=true;
        loginInfo.deviceToken = $localstorage.get('deviceToken');
        loginInfo.version = $localstorage.get('appversion')||'1.1.15';
        loginInfo.eduapp = true; //This flag will differentiate LTA DRIVING and EXAM apps
        if(loginInfo.pass){
          var pass=loginInfo.pass.replace(/\s/g, "")
          loginInfo.hashPass=CryptoJS.MD5(pass).toString(CryptoJS.enc.Hex);
          delete loginInfo.pass;
        }
        return ApiService.post('/login', loginInfo).then(function(result) {
          console.log(result);
          if (result.data.success) {
            return self.getLoginInfo(true).then(function(user) {
              if (user) {
                return self.loginHandler(user);
              } else {
                return result.data;
              }
            });
          } else {
            self.user = null;
            var error = result.data.error || "Login Failed";
            if(error.indexOf("The account is not activated")!=-1){
               error='账户未激活，请重新注册!';
            }
            if(error.indexOf("User not exist")!=-1){
               error='用户名不存在，请进行注册!';
            }
            if(error.indexOf("您的帐号不存在，请自行注册或联系企业管理员完成注册登记。")!=-1){
               error='您的用户名不存在，请自行注册或联系企业管理员完成注册登记。';
            }
            CommonService.showErrorWithTitle($filter('translate')('Error'), $filter('translate')(error));
            return null;
            //return null;
            // if (result.data.errorcode && result.data.errorcode == 1001) { // meaning the use is not actived
            //   deferred.reject(result);
            //   return deferred.promise;
            // } else { // other errors
            //   return null
            // }
          }
        });
      },
      logout: function(flag) {
        var self = this;
        return ApiService.post('/logout').then(function() {
          self.user = null;
          self.profile=null;
          return self.logoutUser(flag);
        });
      },
      loginHandler: function(user) {
        /**
         * Custom logic to manually set userRole goes here
         *
         * Commented example shows an userObj coming with a 'completed'
         * property defining if the user has completed his registration process,
         * validating his/her email or not.
         *
         * EXAMPLE:
         * if (user.hasValidatedEmail) {
         *   wrappedService.userRole = userRoles.registered;
         * } else {
         *   wrappedService.userRole = userRoles.invalidEmail;
         *   $state.go('app.nagscreen');
         * }
         */
        // setup token
        setToken(this.subDomain);
        // update user
        if (!wrappedService.user) {
          wrappedService.user = {};
        }
        angular.extend(wrappedService.user, user);
        // flag true on isLogged
        wrappedService.isLogged = true;
        // update userRole
        if (user.is_admin) {
          wrappedService.userRole = userRoles.super;
        } else if (user.grole == 'admin' || user.is_schoolAdmin) {
          wrappedService.userRole = userRoles.admin;
        } else if (user.grole == 'sysmanager') {
          wrappedService.userRole = userRoles.sysmanager;
        } else if (user.grole == 'sysinspector') {
          wrappedService.userRole = userRoles.sysinspector;
        } else if (user.grole == 'dinspector') {
          wrappedService.userRole = userRoles.dinspector;
        } else if (user.grole == 'dadmin') {
          wrappedService.userRole = userRoles.dadmin;
        } else if (user.grole == 'teacher' || user.is_teacher) {
          wrappedService.userRole = userRoles.teacher;
        } else {
          wrappedService.userRole = userRoles.user;
        }
        return user;
      },
      loginUser: function(user) {
        this.loginHandler(user);
      },
      logoutUser: function(flag) {
        /**
         * De-registers the userToken remotely
         * then clears the loginService as it was on startup
         */
        setToken(null);
        this.userRole = userRoles.public;
      //  this.userchecked=false;
        this.user = null;
        var oldDomain = this.subDomain;
        this.subDomain = null;
        this.school = null;
        $rootScope.currentSchool = null;
        this.isLogged = false;
        this.loggedDomain = false;
        getLoginData(oldDomain);
        if(flag){
          return;
        }
        $state.go(logoutState, {}, {
          reload: true
        });
      },
      setCurrentDomain: function(school) {
        this.school = school;
        this.subDomain = school.key;
        ApiService.setGid(school._id);
      },
      clearCurrentDomain: function(school) {
        this.school = null;
        this.subDomain = null;
        ApiService.setGid(null);
      },
      resolvePendingState: function(httpPromise) {
        var checkUser = $q.defer(),
          self = this,
          pendingState = self.pendingStateChange;
        // When the $http is done, we register the http result into loginHandler, `data` parameter goes into loginService.loginHandler
        httpPromise.success(self.loginHandler);
        httpPromise.then(function success(httpObj) {
          self.doneLoading = true;
          // duplicated logic from $stateChangeStart, slightly different, now we surely have the userRole informations.
          if (pendingState.to.accessLevel === undefined || pendingState.to.accessLevel.bitMask & self.userRole.bitMask) {
            checkUser.resolve();
          } else {
            checkUser.reject('unauthorized');
          }
        }, function reject(httpObj) {
          checkUser.reject(httpObj.status.toString());
        });
        /**
         * I setted up the state change inside the promises success/error,
         * so i can safely assign pendingStateChange back to null.
         */
        self.pendingStateChange = null;
        return checkUser.promise;
      },
      changeDomain:function(){
        getLoginData();
        managePermissions();
      },
      /**
       * Public properties
       */
      userRole: null,
      user: null,
      isLogged: null,
      pendingStateChange: null,
      doneLoading: null,
      subDomain: null,
      school: null,
      loggedDomain: false
    };
    getLoginData();
    managePermissions();
    return wrappedService;
  };
});
